@@ -62,6 +62,8 @@
 
 #define sure_string(s) ((s)!=NULL?(s):"")
 
+extern char **environ;
+
 struct _GnomeDesktopItem {
 	int refcount;
 
@@ -1395,8 +1397,6 @@ sn_error_trap_pop (SnDisplay *display,
 	gdk_error_trap_pop ();
 }
 
-extern char **environ;
-
 static char **
 make_spawn_environment_for_sn_context (SnLauncherContext *sn_context,
 				       char             **envp)
@@ -1825,10 +1825,13 @@ ditem_execute (const GnomeDesktopItem *item,
 			/* The error was set for us,
 			 * we just can't launch this thingie */
 			ret = -1;
+			g_strfreev (real_argv);
 			break;
 		}
 		launched ++;
 
+		g_strfreev (real_argv);
+
 		if (arg_ptr != NULL)
 			arg_ptr = arg_ptr->next;
 
@@ -2930,7 +2933,7 @@ gnome_desktop_item_set_strings (GnomeDesktopItem *item,
 	g_return_if_fail (attr != NULL);
 
 	str = g_strjoinv (";", strings);
-	str2 = g_strconcat (str, ";");
+	str2 = g_strconcat (str, ";", NULL);
 	/* FIXME: there's no way to escape semicolons apparently */
 	set (item, attr, str2);
 	g_free (str);
